MODULE CookSort;
(* Cookie Sorting (to different files) on size(in lines) or rating.
    Sorting Criteria in cooksort.cfg.

  (C) 1998 Marco van de Voort,
  sources Cookie-Project are part of the XTDLIB package. *)

IMPORT FIO,IO;
FROM Str     IMPORT Prepend,Copy,RCharPos,Slice,Append,Caps,Delete,ItemS,
                    CharPos;
FROM DJArc   IMPORT Packer,Unpacker;
FROM Lib     IMPORT Move,CommandLine,Fill;
FROM ELib    IMPORT ChangeExtension,LTrim,SToCard;
FROM MLib    IMPORT FindInPath;
FROM ELib    IMPORT GetSwitch,StripDoubleChar;
FROM Storage IMPORT ALLOCATE;
(*FROM CookShr IMPORT DoConfigLine;
(* Have to change to the generic CategType first *)
*)

CONST
      CrLf      = CHR(13)+CHR(10);
      MaxFileNr = 20;
      Usage=
        " Sorts SRC archives over several files. In normal mode, the size in lines"+CrLf+
        " of the jokes is the sorting-key. When the -r switch is specified, the"+CrLf+
        " category in which the joke falls is the key. See cooksort.cfg for details"+CrLf+CrLf+
        " Switches: "+CrLf+
        " -r      Sort on category. "+CrLf+
        " -c      Copy comments to all files, default is ignore (normal) or dependant of"+CrLf+
        "         configuration (-r mode)"+CrLf+
        " -s      Simple (new) comment to all files, kill all else"+CrLf+
        " -a      Append to existing files if necessary"+CrLf;

TYPE
     Buffer      = ARRAY[0..32767] OF BYTE;
     BufferP     = POINTER TO Buffer;
     FBufType    = ARRAY[0..2047] OF BYTE;
     FBufP       = POINTER TO FBufType;
     FBufArr     = ARRAY[0..MaxFileNr] OF FBufP;

     FIOARR     = ARRAY[0..MaxFileNr] OF FIO.PathStr;
     NameTyp    = POINTER TO FIOARR;
     Set32      = SET OF [0..31];

VAR FIn,FOut1,
    FOut4,FOutMore : FIO.File;
    MainBuf,
    TmpBuf,
    ThisBuf    : BufferP;
    NoRec,
    ThisPnt    : CARDINAL;

    SizeEntr   : CARDINAL;                      (* Size configuration*)
    Names      : NameTyp;
    LineArr    : ARRAY[0..MaxFileNr] OF CARDINAL;

    ValEntr    : CARDINAL;                      (* Value/rating config*)
    SysNam     : NameTyp;
    LoValArr,
    HiValArr   : ARRAY[0..MaxFileNr] OF ARRAY[0..26] OF BYTE;

    FileArr    : ARRAY[0..MaxFileNr] OF FIO.File; (*Filehandles for each file*)
    FBuf       : FBufArr;                         (*buffers for each file*)

    First      : BOOLEAN;
    SpecSort   : BOOLEAN;
    CommentToAll  : BOOLEAN;
    SimpleComment : BOOLEAN;
    AAppend       : BOOLEAN;

PROCEDURE ReadRecord():CARDINAL;

VAR C : CHAR;
    Lfs: CARDINAL;

BEGIN
 ThisPnt:=0; Lfs:=0;
 REPEAT
  C:=FIO.RdChar(FIn);
  ThisBuf^[ThisPnt]:=CHR(C);
  INC(ThisPnt);
 UNTIL C=CHR(10);
 LOOP
  C:=FIO.RdChar(FIn);
  IF C=CHR(1) THEN
   C:=CHR(32);
   END;
  IF C=CHR(13) THEN
   INC(Lfs);
   END;
  IF FIO.EOF THEN
   RETURN Lfs;
   END;
  IF C="<" THEN
    ThisBuf^[ThisPnt]:="<";
    C:=FIO.RdChar(FIn);
    ThisBuf^[ThisPnt+1]:=C;
    IF C="$" THEN
     C:=FIO.RdChar(FIn);
     ThisBuf^[ThisPnt+2]:=C;
     IF C=">" THEN
      INC(ThisPnt,3);
      EXIT
     ELSE
      INC(ThisPnt,3);
      END;
     ELSE
      INC(ThisPnt,2);
      END;
    END;
  ThisBuf^[ThisPnt]:=C;
  INC(ThisPnt);
  END;
 RETURN Lfs;
END ReadRecord;

PROCEDURE WriteRecord(F:FIO.File);

BEGIN
 IF (ThisPnt>10) AND (ThisPnt<32768)THEN
  FIO.WrBin(F,ThisBuf^,ThisPnt);
  END;
END WriteRecord;

PROCEDURE SortSizeRecord(Lfs:CARDINAL);

VAR A : CARDINAL;

BEGIN
IF (ThisBuf^[0]=BYTE("#")) THEN
 IF CommentToAll THEN
  FOR A:=0 TO SizeEntr-1 DO
   WriteRecord(FileArr[A]);
   END;
  END;
 ELSE
  A:=0;
  LOOP
   IF Lfs<=LineArr[A] THEN
    WriteRecord(FileArr[A]);
    EXIT
    END;
   INC(A);
   IF A=SizeEntr THEN
    EXIT
    END;
   END;
  END;
END SortSizeRecord;

PROCEDURE SortDegreeRecord;

VAR A,B,D : CARDINAL;
    C     : BYTE;

BEGIN
 B:=255; C:=9;
 IF ThisBuf^[0]=BYTE("#") THEN
  B:=26;
  END;
 IF ThisBuf^[0]=BYTE("!") THEN
  B:=ORD(ThisBuf^[1])-65;
  IF (ORD(ThisBuf^[2]) >47) AND (ORD(ThisBuf^[2])< 58) THEN
   C:=BYTE(ThisBuf^[1])-BYTE(48);
   END;
  END;
 IF B=255 THEN
  WriteRecord(FileArr[ValEntr-1]); (* Nothing? Then to last file *)
 ELSIF (B=26) AND CommentToAll THEN
  FOR A:=0 TO ValEntr-1 DO
   WriteRecord(FileArr[A]);
   END;
 ELSE
  A:=0;
  LOOP
   IF (LoValArr[A][B] # BYTE(255)) THEN
    IF (C<=HiValArr[A][B]) AND (C>=LoValArr[A][B]) THEN
     WriteRecord(FileArr[A]);
     EXIT
     END;
    END;
   INC(A);
   IF A=(ValEntr) THEN
    IF ThisBuf^[0]#BYTE("#") THEN
     WriteRecord(FileArr[ValEntr-1]);
     END;
    EXIT
    END;
   END;
  END;
END SortDegreeRecord;

VAR Buf1 : ARRAY[0..4095] OF BYTE;

PROCEDURE CompileSRC(ReadFrom:ARRAY OF CHAR);

VAR
    BaseName   : FIO.PathStr;
    Exist      : BOOLEAN;
    Lfs        : CARDINAL;
    A,B        : CARDINAL;

PROCEDURE OpenOutFile(VAR InFile : ARRAY OF CHAR);

VAR L : BOOLEAN;

BEGIN
 L:=FIO.Exists(InFile);
   IF AAppend AND L THEN
    FileArr[A]:=FIO.Append(InFile);
   ELSE
    FileArr[A]:=FIO.Create(InFile);
    END;
   IF SimpleComment THEN
    IF NOT L THEN
     FIO.WrChar(FileArr[A],"#"); FIO.WrStr(FileArr[A],InFile);
     FIO.WrLn(FileArr[A]); FIO.WrStr(FileArr[A],"<$>");
     END;
    END;
END OpenOutFile;

BEGIN
 IO.WrStr("Size Sorting : "); IO.WrStr(ReadFrom); IO.WrLn;
 Copy (BaseName,ReadFrom);
 FIn:=FIO.Open(ReadFrom);
 FIO.AssignBuffer(FIn,Buf1);
 NoRec  :=0;
 IF SpecSort THEN
  B:=ValEntr-1;
 ELSE
  B:=SizeEntr-1;
  END;
 FOR A:=0 TO B DO        (* 3118*)
  IF SpecSort THEN
   OpenOutFile(SysNam^[A]);
  ELSE
   OpenOutFile(Names^[A]);
   END;
  FIO.AssignBuffer(FileArr[A],FBufType(FBuf[A]^));
  END;
 LOOP
  Lfs:=ReadRecord();                            (* Read a Joke*)
  INC(NoRec);
  IF FIO.EOF THEN
   IF SpecSort THEN
    SortDegreeRecord;
   ELSE
    SortSizeRecord(Lfs);
    END;
   EXIT;                                 (* Exit LOOP *)
   END;
  IF SpecSort THEN
   SortDegreeRecord;
  ELSE
   SortSizeRecord(Lfs);
   END;
  END;
 FIO.Close(FIn);
 FOR A:=0 TO SizeEntr-1 DO
 FIO.Close(FileArr[A]);
 END;
END CompileSRC;


PROCEDURE DoConfigLine(VAR Toto : ARRAY OF CHAR);

VAR S : ARRAY [0..100] OF CHAR;
    A,B,Hi,Lo  : CARDINAL;

BEGIN
 A:=0;
 LOOP
  ItemS(S,Toto," ",A);
  IF S[0]=CHR(0) THEN
   EXIT
   END;
  Lo:=0; Hi:=9;
  B:=ORD(S[0])-65;
  IF S[0]="#" THEN
   B:=26;
  ELSIF S[1] # CHR(0) THEN
    IF S[1]="-" THEN
     Hi:=ORD(S[2])-48;
    ELSE
     IF S[2]="-" THEN
      Lo:=ORD(S[1])-48;
      IF S[3]#CHR(0) THEN
       Hi:=ORD(S[3])-48;
       END;
     ELSE
      Hi:=ORD(S[1])-48;
      END;
     END;
  END;
  LoValArr[ValEntr][B]:=BYTE(Lo);
  HiValArr[ValEntr][B]:=BYTE(Hi);
  INC(A);
  END;
END DoConfigLine;

PROCEDURE ReadCfg;

CONST ConfName="cooksort.cfg";

VAR S,S2 : ARRAY[0..127] OF CHAR;
    F    : FIO.File;
    Fl   : BOOLEAN;
    A    : CARDINAL;

BEGIN
 IF FIO.Exists(ConfName) THEN
  Copy(S,ConfName);
 ELSE
  FindInPath(ConfName,S);
  IF S[0]=CHR(0) THEN
   IO.WrStr("Can't find config-file : "); IO.WrStr(ConfName);  IO.WrLn;
   HALT;
   END;
  END;
 F:=FIO.Open(S);
 LOOP
  FIO.RdStr(F,S);
  IF FIO.EOF THEN
   EXIT
   END;
  A:=CharPos(S,";");
  IF A#MAX(CARDINAL) THEN   (* Cut comment*)
   S[A]:=CHR(0);
   END;
  LTrim(S," ");
  Caps(S);
  IF (S[0] #CHR(0)) AND (S[0] # ";") THEN
   IF (S[0]="R") AND (S[1]="E") THEN
    LineArr[SizeEntr]:=MAX(CARDINAL);
    Delete(S,0,4);
    LTrim(S," ");
    Copy(Names^[SizeEntr],S);
    INC(SizeEntr);
    EXIT
    END;
  IF (S[0]="S") AND (S[1]="I") THEN
   Delete(S,0,4);
   LTrim(S," ");
   LineArr[SizeEntr]:=SToCard(S,Fl);
   ItemS(S2,S," ",1);
   Copy(Names^[SizeEntr],S2);
   INC(SizeEntr);
   END;
   END;
  END;
 ValEntr:=0;
 LOOP
  FIO.RdStr(F,S);
  IF FIO.EOF THEN
   EXIT
   END;
  A:=CharPos(S,";");
  IF A#MAX(CARDINAL) THEN   (* Cut comment*)
   S[A]:=CHR(0);
   END;

  LTrim(S," ");
  Caps(S);
  IF (S[0] #CHR(0)) AND (S[0] # ";") THEN
   IF (S[0]="S") AND (S[1]="P") THEN            (* SPEC *)
    Delete(S,0,4);
    LTrim(S," ");
    ItemS(SysNam^[ValEntr],S," ",0);
    A:=CharPos(S," ");
    IF A = MAX(CARDINAL) THEN
     EXIT;
    ELSE
     Delete(S,0,A+1);
     END;
    DoConfigLine(S);
    INC(ValEntr);
   ELSE
    IO.WrStr(" Error in config file "); IO.WrLn;
    HALT;
    END;
   END; (* Check for comment loop *)
  END; (* Loop *)

 IF SizeEntr<2 THEN
  IO.WrStr("Nothing to do, please correct cooksort.cfg"); IO.WrLn;
  HALT;
  END;
 FIO.Close(F);
END ReadCfg;

CONST MainPath="d:\cookie\";

VAR S,S2,S3 :ARRAY[0..127] OF CHAR;
    D : FIO.DirEntry;
    A : CARDINAL;

BEGIN
 SpecSort:=FALSE; CommentToAll:=FALSE; SimpleComment:=FALSE; AAppend:=FALSE;
 FOR A:=0 TO MaxFileNr DO
  NEW (FBuf[A]);
  END;
 NEW (Names); SizeEntr:=0;     NEW(SysNam);
 NEW(MainBuf); NEW(ThisBuf); NEW(TmpBuf);
 Fill(ADR(LoValArr),SIZE(LoValArr),255);  (* 255= entry unused *)
 Fill(ADR(HiValArr),SIZE(HiValArr),255);
 IO.WrStr("CookSort V1.02, (C) 1998 by Marco van de Voort"); IO.WrLn;IO.WrLn;
 ReadCfg;

 Copy (S,CommandLine^);
 WHILE GetSwitch (S,S3,"-") DO
  CASE CAP(S3[0]) OF
   |"R" : SpecSort:=TRUE;
   |"C" : CommentToAll:=TRUE;
   |"S" : SimpleComment:=TRUE;
          FOR A:=0 TO ValEntr-1 DO
           LoValArr[A][26]:=255;
           HiValArr[A][26]:=255;
           END;
   |"A" : AAppend:=TRUE;

  ELSE
   IO.WrStr(" Unknow switch : ");
   IO.WrStr(S3); IO.WrLn;IO.WrStr(Usage); HALT;
   END;
  END;
 IF SimpleComment THEN
  CommentToAll:=FALSE;
  END;
 StripDoubleChar(S," ");
 LTrim(S," ");
 IF RCharPos(S,"*") # MAX(CARDINAL) THEN
  A:=RCharPos(S,"\");
  Slice(S2,S,0,A+1);
  IF FIO.ReadFirstEntry(S,FIO.FileAttr{},D) THEN
   REPEAT
    Copy(S3,S2);
    Append(S3,D.Name);
    CompileSRC(S3);
    UNTIL NOT FIO.ReadNextEntry(D);
   ELSE
    IO.WrStr("File not found"+CrLf);
    IO.WrStr(Usage);
    END;
 ELSE
  IF NOT FIO.Exists(S) THEN
   Prepend(S,MainPath);
   END;
  IF FIO.Exists(S) THEN
   CompileSRC(S);
   IO.WrStr("Compiled "); IO.WrCard(NoRec,0); IO.WrStr(" record"); IO.WrLn;
  ELSE
   IO.WrStr("File not found"+CrLf);
   IO.WrStr(S);
   IO.WrStr(Usage);
   END;
  END;
END CookSort.