IMPLEMENTATION MODULE Tlib;

FROM Storage IMPORT
     (* procs *) ALLOCATE, DEALLOCATE;

FROM Lib     IMPORT GetDate,GetTime,DayType;
FROM CoreCrt IMPORT GotoXY,WhereX,WhereY,CursorOn,CursorOff,CursorOverwrite,
                    ReturnCurSize,SetCurSize;
FROM ELib    IMPORT GetKey,CardToS,Months,Dows,SToCard,DaTiToStr;

IMPORT Crt, FIO, Str, IO;

(***************************************************************************)
(* Keyboard Routines                                                       *)
(*                                                                         *)
(* Author : A.T. Reinicke                                                  *)
(* for    : Beton Son B.V. t.b.v. afd. Automatisering                      *)
(* Date   : 20-04-1990                                                     *)
(***************************************************************************)
(* Small corrections and adoptions to XtdLib libraries(all duplicate       *)
(* procedures removed) by Marco van de Voort during 1996                   *)
(* All TLib routines, my corrections incluis (C) A. Reinicke  1990-1996    *)
(* All XtdLib procedures (CoreCrt,ELib,Crt) (C) Marco van de Voort 1996    *)
(***************************************************************************)

CONST
   NormalColor  = 0+ 0+0+7;
   HilightColor = 0+16+0+7;
   EditColor    = 0+16+8+7;

VAR
   ins : BOOLEAN;

PROCEDURE InputStr( VAR S            : ARRAY OF CHAR;
                        X, Y, Length : CARDINAL;
                        OptionSet    : BOOLEAN;
                    VAR Changed      : BOOLEAN ) : CARDINAL;
VAR
   TmpStr  : ARRAY [0..255] OF CHAR;
   k, p, l : CARDINAL;
   A       : BYTE;
BEGIN
   A:=Crt.TColor;
   Crt.TextColor(Crt.Color(EditColor));
   IF OptionSet THEN
      Str.Copy( TmpStr, S );
   ELSE
      TmpStr[0] := CHR(NUL);
   END;
   p := 0;
   Changed := FALSE;
   LOOP
      l := Str.Length( TmpStr );
      IF p > l THEN
         p := l;
      END;
      Crt.DirectWrite(X,Y,ADR(TmpStr), Length );
      GotoXY( X+p, Y );
      IF ins THEN
         CursorOn;
      ELSE
         CursorOverwrite;
      END;
      k := GetKey();
      CursorOff;
      CASE k OF
      | Space..Tilde : IF ins THEN
                          Str.Insert( TmpStr, CHAR(k), p );
                       ELSIF p = l THEN
                          Str.Append( TmpStr, CHAR(k) );
                       ELSE
                          TmpStr[p] := CHAR(k);
                       END;
                       INC( p );
      | Home         : p := 0;
      | End          : p := l;
      | ArrL         : IF p > 0 THEN
                          DEC( p );
                       END;
      | ArrR         : IF p < l THEN
                          INC( p );
                       END;
      | Delete       : IF p < l THEN
                          Str.Delete( TmpStr, p, 1 );
                       END;
      | BS           : IF p > 0 THEN
                          DEC( p );
                          Str.Delete( TmpStr, p, 1 );
                       END;
      | Insert       : ins := NOT ins;
      | CtrlY        : IF p < l THEN
                          TmpStr[p] := CHR(NUL);
                       END;
      | CR, ArrU, ArrD : EXIT;
      | ESC          : Changed := FALSE;
                       EXIT;
      END;
      IF k # ESC THEN
         Changed := TRUE;
      END;
   END;
   IF Str.Compare( TmpStr, S ) = 0 THEN
      Changed := FALSE;
   END;
   IF Changed THEN
      Str.Copy( S, TmpStr );
   END;
   Crt.DirectWrite( X, Y, ADR(S),Length);
   Crt.TextColor(Crt.Color(A));
   RETURN k;
END InputStr;

PROCEDURE Selector( Init : ProcI; Display : ProcD; MaxLines, Lines : CARDINAL; VAR First, Curr : CARDINAL ) : CARDINAL;
VAR
   Key, Entry : CARDINAL;

   PROCEDURE DisplayLines( First, Lines : CARDINAL );
   VAR
      Line : CARDINAL;
   BEGIN
      FOR Line := First TO First+Lines-1 DO
         Display( Line, Line-First, NormalColor );
      END;
   END DisplayLines;

BEGIN
   CursorOff;
   Init;
   DisplayLines( First, Lines );
   Entry := Curr;
   LOOP
      Display( Entry, Entry-First, HilightColor );
      Key := GetKey();
      Display( Entry, Entry-First, NormalColor );
      CASE Key OF
      | ArrD : IF Entry < MaxLines THEN
                  INC( Entry );
               ELSE
                  Entry := 1;
               END;
      | ArrU : IF Entry > 1 THEN
                  DEC( Entry );
               ELSE
                  Entry := MaxLines;
               END;
      | PgDn : IF Entry < First+Lines-1 THEN
                  Entry := First+Lines-1;
                  IF Entry > MaxLines THEN
                     Entry := MaxLines;
                  END;
               ELSIF Entry+Lines > MaxLines THEN
                  Entry := MaxLines;
               ELSE
                  INC( Entry, Lines );
               END;
      | PgUp : IF Entry > First THEN
                  Entry := First;
               ELSIF INTEGER(Entry)-INTEGER(Lines) < 1 THEN
                  Entry := 1;
               ELSE
                  DEC( Entry, Lines );
               END;
      | Home : Entry := 1;
      | End  : Entry := MaxLines;
      | Insert, Delete, CR, F1 :
               Curr := Entry;
               EXIT;
      | ESC  : Curr := 0;
               EXIT;
      END;
      IF Entry < First THEN
         First := Entry;
         DisplayLines( First, Lines );
      ELSIF Entry > First+Lines-1 THEN
         First := Entry-Lines+1;
         DisplayLines( First, Lines );
      END;
   END;
   CursorOn;
   RETURN Key;
END Selector;

PROCEDURE EditRecord( Init : ProcI; Display : ProcD; Edit : ProcE; MaxField : CARDINAL );
VAR
   Key, Field : CARDINAL;
BEGIN
   CursorOff;
   Init;
   FOR Field := 1 TO MaxField DO
      Display( Field, 0, NormalColor );
   END;
   Field := 1;
   LOOP
      Display( Field, 0, HilightColor );
      Key := GetKey();
      Display( Field, 0, NormalColor );
      CASE Key OF
      | ArrD : IF Field < MaxField THEN
                  INC( Field );
               ELSE
                  Field := 1;
               END;
      | ArrU : IF Field > 1 THEN
                  DEC( Field );
               ELSE
                  Field := MaxField;
               END;
      | PgDn : Field := MaxField;
      | PgUp : Field := 1;
      | CR   : Key := Edit( Field );
      | ESC  : EXIT;
      END;
   END;
   CursorOn;
END EditRecord;

(***************************************************************************)
(* Graphic Percentage Bars                                                 *)
(***************************************************************************)

TYPE
   BarType = POINTER TO BarTypeStructure;
   CharArray = POINTER TO ARRAY [0..255] OF CHAR;
   BarTypeStructure = RECORD
      Size, Xpos, Ypos, Res, LastGrade : CARDINAL;
      Chars                            : CharArray;
   END;

PROCEDURE DefineBar( VAR Bar              : BarType;
                         Size, Xpos, Ypos : CARDINAL;
                         Chars            : ARRAY OF CHAR );
VAR
   I : CARDINAL;
BEGIN
   IF (Str.Length(Chars) > 1) AND (Str.Length(Chars) < 256) THEN
      NEW( Bar );
      Bar^.Size := Size;
      IF Xpos = MAX(CARDINAL) THEN
         Bar^.Xpos := WhereX();
      ELSE
         Bar^.Xpos := Xpos;
      END;
      IF Ypos = MAX(CARDINAL) THEN
         Bar^.Ypos := WhereY();
      ELSE
         Bar^.Ypos := Ypos;
      END;
      Bar^.Res  := Str.Length(Chars)-1;
      Bar^.LastGrade := 0;
      ALLOCATE( Bar^.Chars, Bar^.Res+1 );
      FOR I := 0 TO Bar^.Res DO
         Bar^.Chars^[I] := Chars[I];
      END;
   ELSE
      Bar := NIL;
   END;
END DefineBar;

PROCEDURE UpdateBar(     Bar              : BarType;
                         Length, Position : LONGCARD );
VAR
   FSize, BSize, Xtemp, Ytemp, Grade, MC, OldCursor : CARDINAL;
   Mchar                                            : CHAR;
BEGIN
   IF Bar # NIL THEN
      Grade := CARDINAL(FLOAT(Position)/FLOAT(Length)*FLOAT(LONGCARD(Bar^.Size)*LONGCARD(Bar^.Res)));
      FSize := Grade DIV Bar^.Res;
      MC := Grade MOD Bar^.Res;
      IF MC = 0 THEN
         BSize := Bar^.Size-FSize;
         Mchar := '';
      ELSE
         BSize := Bar^.Size-FSize-1;
         Mchar := Bar^.Chars^[MC];
      END;
      IF Grade # Bar^.LastGrade THEN
         Xtemp := WhereX();
         Ytemp := WhereY();
         OldCursor := ReturnCurSize();
         GotoXY( Bar^.Xpos, Bar^.Ypos );
         IO.WrCharRep( Bar^.Chars^[Bar^.Res], FSize );
         IO.WrChar( Mchar );
         IO.WrCharRep( Bar^.Chars^[0], BSize );
         Bar^.LastGrade := Grade;
         GotoXY( Xtemp, Ytemp );
         SetCurSize( OldCursor AND 255 ,OldCursor>>8);
      END;
   END;
END UpdateBar;

PROCEDURE RemoveBar( VAR Bar : BarType );
BEGIN
   IF Bar # NIL THEN
      DEALLOCATE( Bar^.Chars, Bar^.Res+1 );
      DISPOSE( Bar );
   END;
END RemoveBar;

(***************************************************************************)
(* Logging (using LogFiles)                                                *)
(***************************************************************************)

PROCEDURE LogDate( F : FIO.File );
VAR
   Year, Month, Day : CARDINAL;
   Dow              : DayType;
   DateString,
   LogString        : ARRAY [0..13] OF CHAR;

BEGIN
   GetDate( Year, Month, Day, Dow );
   Str.Copy(LogString,Dows[CARDINAL(Dow)]);
   DaTiToStr( 0,0,0,Day,Month,Year, " %d %s %y", DateString );
   Str.Append( LogString, DateString );
   FIO.WrStr( F, LogString );
END LogDate;

PROCEDURE LogTime( F : FIO.File );
VAR
   Hour, Min, Sec, Hsec : CARDINAL;
   LogString            : ARRAY [0..8] OF CHAR;
BEGIN
   GetTime( Hour, Min, Sec, Hsec );
   DaTiToStr( Hour, Min, Sec,0,0,0,"%h:%m:%s", LogString );
   FIO.WrStr( F, LogString );
END LogTime;

PROCEDURE OpenLog( Name : ARRAY OF CHAR ) : FIO.File;
VAR
   F : FIO.File;
BEGIN
   IF FIO.Exists( Name ) THEN
      F := FIO.Append( Name );
   ELSE
      F := FIO.Create( Name );
   END;
   RETURN F;
END OpenLog;

PROCEDURE CloseLog( F : FIO.File );
BEGIN
   FIO.Close( F );
END CloseLog;

PROCEDURE LogHeader( LogFile, ProgID : ARRAY OF CHAR );
VAR
   F : FIO.File;
BEGIN
   F := OpenLog( LogFile );
   FIO.WrLn( F );
   FIO.WrStr( F, "----------  " );
   LogDate( F );
   FIO.WrStr( F, ", " );
   FIO.WrStr( F, ProgID );
   FIO.WrLn( F );
   CloseLog( F );
END LogHeader;

PROCEDURE LogEntry( LogFile, LogText : ARRAY OF CHAR; Level : CHAR );
VAR
   F : FIO.File;
BEGIN
   IF Level IN LogLevels THEN
      F := OpenLog( LogFile );
      FIO.WrChar( F, Level );
      FIO.WrChar( F, CHR(Space) );
      LogTime( F );
      FIO.WrStr( F, "  " );
      FIO.WrStr( F, LogText );
      FIO.WrLn( F );
      CloseLog( F );
   END;
END LogEntry;

(***************************************************************************)
(* Diversen                                                                *)
(***************************************************************************)

VAR
   Info : RECORD
      Length : CARDINAL;
      Line1a,
      Line1b,
      Line2a,
      Line2b : ARRAY [0..79] OF CHAR;
   END;

PROCEDURE SetProgramInfo( Len : CARDINAL; Name, Version, Date,
                          Programmer, Company : ARRAY OF CHAR   );
VAR
   SubStr : ARRAY [0..3] OF CHAR;
   Num    : CARDINAL;
   S      : ARRAY [0..8] OF CHAR;
   Status : BOOLEAN;
BEGIN
   Info.Length := Len;
   Str.Copy( Info.Line1a, Name );
   Str.Copy( Info.Line1b, Programmer );
   Str.Copy( Info.Line2a, "version " );
   Str.Append( Info.Line2a, Version );
   Str.Append( Info.Line2a, " (" );
   Str.Slice( SubStr, Date, 0, 2 );
   Num := SToCard( SubStr, Status ) ;
   Str.Append(Info.Line2a,Months[Num]);
   Str.Append( Info.Line2a, ", " );
   Str.Slice( SubStr, Date, 3, 2 );
   Num := SToCard( SubStr,  Status ) ;
   Status:=CardToS(Num, S );
   Str.Append( Info.Line2a, S );
   CASE Num OF
   | 1, 21, 31 :
      Str.Append( Info.Line2a, "st " );
   | 2, 22 :
      Str.Append( Info.Line2a, "nd " );
   | 3, 23 :
      Str.Append( Info.Line2a, "rd " );
   ELSE
      Str.Append( Info.Line2a, "th " );
   END;
   Str.Slice( SubStr, Date, 6, 4 );
   Str.Append( Info.Line2a, SubStr );
   Str.Append( Info.Line2a, ") " );
   Str.Copy( Info.Line2b, Company );
END SetProgramInfo;

PROCEDURE WriteProgramInfo();
BEGIN
   IO.WrCharRep( CHR(205), Info.Length );
   IO.WrLn;
   IO.WrStr( Info.Line1a );
   IO.WrStrAdj( Info.Line1b, Info.Length-Str.Length( Info.Line1a ) );
   IO.WrLn;
   IO.WrStr( Info.Line2a );
   IO.WrStrAdj( Info.Line2b, Info.Length-Str.Length( Info.Line2a ) );
   IO.WrLn;
   IO.WrCharRep( CHR(205), Info.Length );
   IO.WrLn;
   IO.WrLn;
END WriteProgramInfo;

BEGIN
   ins := TRUE;
END Tlib.