MODULE Dict3;
(* Dict3 (C) by Marco van de Voort 1996
   The entire program requires VGA.

   Dict is a abbreviation for Dictatuur(dutch for Dictatorship). It's a form
   of the game of Life.

   The reason that the particular version I tried to reproduce here was
   called Dictatorship, was that the original version of procedure DoStuff
   was different, and the screen it converted was not convergent, neither
   color took control, this was called Democracy. By a small change in the
   algoritm of DoStuff one  of the colors filled the screen at the end,
   like in a Dictatorship.

   There are 4 versions

   The difference between Dict and Dict2 is the handling of the borders of the
   screen, in Dict the border units do not have neighbours outside the screen,
   so these units do not have 8 neigbours but 5, or even 3 (in the corners).
   In Dict2 the border-units of the screen are fixed on the initial colors,
   which makes colorsurface 'stick' on the walls.

   Dict is also a demonstration of the virtual screen.

   Dict3 and Dict4 are graphical version of resp. Dict and Dict2, and therefore
   Dict3 and Dict4 have a higher resolution, but are much slower.
   I recommend to compile Dict3 and Dict4 with a coprocessor on.
   *)

FROM CoreCrt IMPORT Red,White,Black,Color,ClrScr,ChangeMode;
FROM Lib IMPORT RANDOMIZE,RANDOM;
FROM IO  IMPORT RdKey,KeyPressed,WrLngCard,WrStr,WrLn;
FROM Graph IMPORT InitVGA,Plot,Point,TextMode,GraphMode;
FROM ELib  IMPORT GetKey;

IMPORT Crt;
CONST WhiteCrit=2;
      RedCrit  =2;
      RandWhiteCrit=2;
      RandRedCrit  =2;
      Width    = 640;
      Depth    = 479;

PROCEDURE DoGeneration;

VAR A,B,C,
    CWhite,CRed : CARDINAL;

PROCEDURE Process(Value:CARDINAL);

BEGIN
 IF Value   =ORD(Red) THEN
  INC(CRed);
 ELSIF Value=ORD(White) THEN
  INC(CWhite);
  END;
END Process;

PROCEDURE DoStuff(CritW,CritR:CARDINAL);

BEGIN
 CWhite:=0;
 CRed  :=0;
 IF A#0 THEN
  IF B #0 THEN
   Process(ORD(Point(B-1,A-1)));
   END;
  Process(ORD(Point(B  ,A-1)));
  IF B # (CARDINAL(Width)-1) THEN
   Process(ORD(Point(B+1,A-1)));
   END;
  END;
 IF B#0 THEN
  Process(ORD(Point(B-1,A  )));
  END;
 IF B # (CARDINAL(Width)-1) THEN
  Process(ORD(Point(B+1,A  )));
  END;
 IF A # Depth THEN
  IF B#0 THEN
   Process(ORD(Point(B-1,A+1)));
   END;
  Process(ORD(Point(B  ,A+1)));
  IF B # (CARDINAL(Width)-1) THEN
   Process(ORD(Point(B+1,A+1)));
   END;
  END;

 IF (CWhite>CritW) AND (CRed>CritR) THEN
  C:=RANDOM(2);
  IF C=1 THEN
   Plot(B,A,ORD(Red));
  ELSE
   Plot(B,A,ORD(White));
   END;
 ELSIF ((CWhite<CritW) AND (CRed<CritR)) THEN

 ELSE
  IF CWhite>CritW THEN
   Plot(B,A,ORD(White));
   END;
  IF CRed  >CritR THEN
   Plot(B,A,ORD(Red));
   END;
  END;
END DoStuff;

BEGIN
 FOR A:=0 TO CARDINAL(Depth) DO
  FOR B:=0 TO CARDINAL(Width)-1 DO
   IF (B=0) AND ((A>0) AND (A<CARDINAL(Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B=CARDINAL(Width)-1) AND ((A>0) AND (A<CARDINAL(Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B<CARDINAL(Width)-1) AND ((B>0) AND (A=CARDINAL(Depth)) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF (B<CARDINAL(Width)-1) AND ((B>0) AND (A=0) ) THEN
    DoStuff(RandWhiteCrit,RandRedCrit );
    END;
   IF ((B>0) AND (B<CARDINAL(Width)-1)) AND ((A>0) AND (A<CARDINAL(Depth))) THEN
    DoStuff(WhiteCrit,RedCrit);
    END;
   END;
  END;

END DoGeneration;

VAR A,X,Y : CARDINAL;
    C     : Color;
    Nr,Wh,Re,Bl    : LONGCARD;


BEGIN

 InitVGA;
 GraphMode;
 RANDOMIZE;
 FOR Y:=0 TO Depth DO
  FOR X:= 0 TO Width-1 DO
   A:=RANDOM(2);
   IF A=1 THEN
    C:=Red;
   ELSE
    C:=White;
    END;
   Plot(X,Y,ORD(C));
   END;
  END;
 Nr:=0;
 Bl:=0;
 Wh:=0;
 Re:=0;
 WHILE NOT KeyPressed() DO
  DoGeneration;
  INC(Nr);
  END;
 FOR Y:=0 TO Depth DO
  FOR X:= 0 TO Width-1 DO
   A:=CARDINAL(Point(X,Y));
   IF A=ORD(Red) THEN
    INC(Re);
   ELSIF A=ORD(White) THEN
    INC(Wh);
   ELSIF A=ORD(Black) THEN
    INC(Bl);
    END;
   END;
  END;
 TextMode;
 ChangeMode(3);
 WrStr("Nr of tests : ");
 WrLngCard(Nr,8);
 WrLn;
 WrStr("White parts : ");
 WrLngCard(Wh,8);
 WrLn;
 WrStr("Red   parts : ");
 WrLngCard(Re,8);
 WrLn;
 WrStr("Black parts : ");
 WrLngCard(Bl,8);
 WrLn;
 A:=GetKey();
END Dict3.
