MODULE DoRed;
(* An useful utility, changes a directory into another in all *.red files
 in this directory and below. NEW: Remove the colors from the libraries.
 A demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1997

 DirTree is NOT dependant on XtdLib, this demo however is, but slightly.

See constant Usage for more information.

This source is also a nice example of the use of some typical often used
 XtdLib procedures like ExpPath,ChangeExtension and Replace.
*)

FROM Lib  IMPORT ParamCount,ParamStr,CommandLine;
FROM ELib IMPORT ExpPath,ChangeExtension;
FROM Str  IMPORT Concat,Caps,Copy,Match;
FROM MLib IMPORT Replace;
IMPORT IO,FIO,Crt,DirTree;

CONST CR=CHR(13)+CHR(10);

        Usage=

 "Process all *.red files in current directory and below, changing"+CR+
 " the 1st parameter( a directory, e.g. 'C:\TS') into the"         +CR+
 " 2nd parameter (also a directory D:\PROG\TS)"+                    CR+
 "The program keeps .BAK files for safety reasons, use del *.bak /s to delete."+
                                                                 CR+CR+
 "Written for XtdLib, to change source locations and compiler directories,"+
                                                                           CR+
 " but I also use it for packages like PMOS."+CR+
 CR+
 "Usage : DoRed <old path> <new path>"+CR+
 "      E.g. DoRed D:\PROG\TS C:\TS     or"+CR+
 "           DoRed d:\prog\ts ..\utils\programming\m2\topspeed\ts31"+CR+
                                                                        CR+
 "*new* or DoRed /T      to recursively kill all colors in .DEFs and .MODs in"+CR+
 "                      this directory and below"+CR;


VAR CurDir,
    Param1,Param2 : ARRAY[0..127] OF CHAR; (* Size string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)

    Fl            : BOOLEAN;

PROCEDURE KillColors(Dir:FIO.PathStr;Fil:FIO.DirEntry);

VAR F,F2     : FIO.File;
    B,B2     : ARRAY[0..4095] OF BYTE;
    C        : CHAR;
    S,S2,S3  : FIO.PathStr;
    A        : CARDINAL;

BEGIN
 Concat(S2,Dir,Fil.Name);
 Copy(S3,S2);
 ChangeExtension(S3,"TMP");
 IF NOT FIO.Exists(S2) THEN
  RETURN;
  END;
 F:=FIO.Open(S2);
 IO.WrStr("Processing ");
 IO.WrStr(S2);
 IO.WrLn;
 F2:=FIO.Create(S3);
 FIO.AssignBuffer(F,B);         FIO.AssignBuffer(F2,B2);
 LOOP
  C:=FIO.RdChar(F);
  IF C>CHR(4) THEN
   FIO.WrChar(F2,C);
   END;
  IF FIO.EOF THEN
   EXIT
   END;
  END;
 FIO.Close(F);
 FIO.Close(F2);
 Copy(S,S2);
 ChangeExtension(S,"BAK");
 IF FIO.Exists(S) THEN
  FIO.Erase(S);
  END;
 FIO.Rename(S2,S);
 FIO.Rename(S3,S2);
END KillColors;

PROCEDURE ChangeRedirection(Dir:FIO.PathStr;Fil:FIO.DirEntry);

VAR S,S2,S3 : ARRAY[0..100] OF CHAR;
    F,F2 : FIO.File;
    Read : ARRAY[0..300] OF CHAR;
    A : CARDINAL;

BEGIN
 Crt.CrtOff; (* Direct screenwrites off, dos writes on, output now pipable*)
 IO.WrStr("Processing : ");
 Concat(S,Dir,Fil.Name);
 IO.WrStrAdj(S,-40); IO.WrLngCard(Fil.size,15); IO.WrLn;
 Copy(S2,S);
 F:=FIO.Open(S);
 ChangeExtension(S2,"TMP");
 F2:=FIO.Create(S2);
 LOOP
  FIO.RdStr(F,Read);
  IF FIO.EOF THEN
   EXIT;
   END;
  Caps(Read);
  Replace(Read,Param1,Param2);
  Caps(Read);
  FIO.WrStr(F2,Read); FIO.WrLn(F2);
  END;
 FIO.Close(F);
 FIO.Close(F2);
 Crt.CrtOn; (* DSWs on again *)
 Copy(S3,S2);
 ChangeExtension(S3,"BAK");
 IF FIO.Exists(S3) THEN
  FIO.Erase(S3);
  END;
 FIO.Rename(S,S3);
 FIO.Rename(S2,S);

END ChangeRedirection;

VAR A: CARDINAL;

BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("DoRed 0.002, (C) Marco van de Voort, XtdLib package");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn;
 IO.WrLn;
 FIO.GetDir(0,CurDir);
 DirTree.DirsToo:=TRUE;
 Caps(CommandLine^);
 IF Match(CommandLine^,"/T") THEN
  Crt.CrtOn;
  DirTree.FileScan(CurDir,"*.MOD",KillColors);
  DirTree.FileScan(CurDir,"*.DEF",KillColors);
  Crt.CrtOff; (* Next output should be pipable, normal output*)
 ELSE

  IF ParamCount() <2 THEN
    Crt.CrtOff;
    IO.WrStr(Usage);
    HALT;
    END;
  ParamStr(Param1,1); ParamStr(Param2,2);

  Fl:=ExpPath(Param1); (* Simplify the path, ..\ .\ etc are expanded,
                          makes scan a lot more robust, ExpPath has
                          however some theoretical drawbacks, see ELib.DEF.*)
  Caps(Param1); Caps(Param2);
  Crt.CrtOn;
  DirTree.FileScan(CurDir,"*.red",ChangeRedirection);
  Crt.CrtOff; (* Next output should be pipable, normal output*)
  END;
 IO.WrCharRep("-",78);
 IO.WrLn;
 IO.WrCard(DirTree.FoundCount,35); IO.WrStr(" files, ");
 IO.WrLngCard(DirTree.TotalBytes,12); IO.WrStr(" bytes");
 IO.WrLn; IO.WrLn;
END DoRed.