MODULE TreeDemo;
(* A demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1996

1.04 : A little improved, but with great consequences. More than two commandline
        arguments supported and correct statistics for this situation.
        The current dir isn't included in the statistics about counted dirs.

BuildTree/(multiple) SearchForFiles/ ScanFile routines are extremely powerfull.
The only problem is their memory consumption. For several thousand files, you
easily need some 100-300k of heap-memory free.

The real special and nice possibilities are not demonstrated yet, see help
of SearchForFiles for help.
*)

FROM Str     IMPORT Copy,Append;
FROM DirTree IMPORT BuildTree,DirTreePoint,FilePtr,SearchForFiles,ScanTree;
FROM Storage IMPORT ALLOCATE,DEALLOCATE,HeapAvail,MainHeap;
FROM Lib     IMPORT ParamCount,ParamStr;
FROM ELib    IMPORT ExpPath;

IMPORT IO,FIO,Crt,DirTree;

PROCEDURE DisplayFile(S:ARRAY OF CHAR;DirE:FIO.DirEntry);

BEGIN
 IO.WrStr(S); IO.WrStr(DirE.Name); IO.WrLn;
END DisplayFile;

PROCEDURE Usage;

BEGIN
 IO.WrStr("Searches a directory and below for matching files.");
 IO.WrLn;
 IO.WrStr("By building a tree-structure in memory");
 IO.WrLn;
 IO.WrStr("Usage : TreeDemo <Directory> <FileSpec> [<filespec>]"); IO.WrLn;
 IO.WrStr("      E.g. TreeDemo c:..\xtdlib *.def *.mod *.a");
 IO.WrLn; IO.WrLn;
END Usage;

VAR Root : DirTreePoint;
    Param1,Param2 : ARRAY[0..127] OF CHAR; (* Size string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)
    Fl            : BOOLEAN;
    A             : CARDINAL;
    Files,Dirs    : CARDINAL;
    Bytes:LONGCARD;
    MemStartUp    : LONGCARD;
BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("TreeDemo 1.0, (C) Marco van de Voort, XtdLib package");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn; IO.WrLn;

 IF ParamCount() <2 THEN
   Usage;
   HALT;
   END;
 ParamStr(Param1,1); ParamStr(Param2,2);
 IF NOT FIO.Exists(Param1) THEN
  Crt.TextColor(Crt.LightRed);
  IO.WrStr("Error : Path doesn't exist."); IO.WrLn;
  HALT
  END;
 Crt.CrtOff;
 MemStartUp:=LONGCARD(HeapAvail(MainHeap))*16;
 DirTree.DirsToo:=TRUE;
 Fl:=ExpPath(Param1); (* Simplify the path, ..\ .\ etc are expanded,
                         makes scan a lot more robust, ExpPath has
                         however some theoretical drawbacks, see ELib.DEF.*)
 Crt.CrtOn;
 Root:=BuildTree(Param1);                (* Build Directory tree in memory
                                            (SO ONLY DIRECTORIES!) *)
 Files:=0; Bytes:=0; Dirs:=0;
 FOR A:=2 TO ParamCount() DO
  ParamStr(Param2,A);
  SearchForFiles(Root,Param2,NULLPROC);          (* add files selectively (e.g. *.mod) to
                                           the tree*)
  Crt.CrtOff; (* Next output should be pipable, normal output*)

  INC(Files,DirTree.FoundCount);
  INC(Bytes,DirTree.TotalBytes);

  END;
 ScanTree(Root,DisplayFile,NULLPROC);              (* Display found files with complete
                                                   path*)
 MemStartUp:=MemStartUp-(LONGCARD(HeapAvail(MainHeap))*16);
 DirTree.KillFileTree(Root);


 IO.WrCharRep("-",78);
 IO.WrLn;
 IO.WrCard(Files,25); IO.WrStr(" files, ");
 IO.WrLngCard(Bytes,12); IO.WrStr(" bytes in ");
 IO.WrCard(DirTree.FoundDirs,0); IO.WrStr(" directories");
 IO.WrLn; IO.WrLn;
 IO.WrStr("Files and directories occupied "); IO.WrLngCard(MemStartUp,0); IO.WrStr(" bytes of memory");
 IO.WrLn;
END TreeDemo.