IMPLEMENTATION MODULE Tlib;

IMPORT SYSTEM, Lib, Str, IO, Storage;

(***************************************************************************)
(* Keyboard Routines                                                       *)
(*                                                                         *)
(* Definitiemodule voor toetsenbord-scancodes & Printer-besturingstekens.  *)
(*                                                                         *)
(* Auteur : A.T. Reinicke                                                  *)
(* Voor   : Beton Son B.V. t.b.v. afd. Automatisering                      *)
(* Datum  : 20-04-1990                                                     *)
(***************************************************************************)

PROCEDURE GetKey() : CARDINAL ;
VAR
   c : CARDINAL;
BEGIN
   c := CARDINAL(IO.RdKey());
   IF c = NUL THEN
      c := CARDINAL(IO.RdKey());
      RETURN c<<8;
   ELSE
      RETURN c;
   END;
END GetKey;

PROCEDURE WhereX() : CARDINAL;
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 0FH;                               (* Get Active display page *)
   Lib.Intr( Reg, 0010H );
   Reg.AH := 03H;                                  (* Read cursor position *)
   Lib.Intr( Reg, 0010H );
   RETURN CARDINAL(Reg.DL);
END WhereX;

PROCEDURE WhereY() : CARDINAL;
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 0FH;                               (* Get active display page *)
   Lib.Intr( Reg, 0010H );
   Reg.AH := 03H;                                  (* Read cursor position *)
   Lib.Intr( Reg, 0010H );
   RETURN CARDINAL(Reg.DH);
END WhereY;

PROCEDURE GotoXY( X, Y : CARDINAL );
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 0FH;                               (* Get active display page *)
   Lib.Intr( Reg, 0010H );
   Reg.AH := 02H;                                   (* Set cursor position *)
   Reg.DL := SHORTCARD(X);
   Reg.DH := SHORTCARD(Y);
   Lib.Intr( Reg, 0010H );
END GotoXY;

PROCEDURE ClrEol;
VAR
   x, y : CARDINAL;
BEGIN
   x := WhereX();
   y := WhereY();
   IO.WrCharRep( CHR(Space), 79-x );
   GotoXY( x, y );
END ClrEol;

PROCEDURE InputStr( VAR S            : ARRAY OF CHAR;
                        X, Y, Length : CARDINAL;
                        OptionSet    : BOOLEAN;
                    VAR Changed      : BOOLEAN ) : CARDINAL;
VAR
   ins     : BOOLEAN;
   k, p, l : CARDINAL;
BEGIN
   GotoXY( X, Y );
   IF OptionSet THEN
      p := Str.Length( S );
   ELSE
      p := 0;
      S[0] := CHR(NUL);
   END;
   ins := TRUE; (* Insert mode *)
   LOOP
      l := Str.Length(S);
      IF p>l THEN
         p := l;
      END;
      IO.WrStr( S );
      ClrEol;
      GotoXY( X+p, Y );
      k := GetKey();
      CASE k OF
      | Space..Tilde : IF ins THEN
                          Str.Insert( S, CHAR(k), p );
                       ELSIF p=l THEN
                          Str.Append( S, CHAR(k) );
                       ELSE
                          S[p] := CHAR(k);
                       END;
                       INC( p );
      | Home         : p := 0;
      | End          : p := l;
      | ArrL         : IF p>0 THEN
                          DEC( p );
                       END;
      | ArrR         : IF p<l THEN
                          INC( p );
                       END;
      | Delete       : IF p<l THEN
                          Str.Delete( S, p, 1 );
                       END;
      | BS           : IF p>0 THEN
                          DEC( p );
                          Str.Delete( S, p, 1 );
                       END;
      | Insert       : ins := NOT ins;
      | CtrlY        : IF p<l THEN
                          S[p] := CHR(NUL);
                       END;
      | CR           : RETURN k;
      | ESC          : RETURN k;
      END;
      GotoXY( X, Y );
   END;
END InputStr;

(***************************************************************************)
(* MS-DOS Routines                                                         *)
(***************************************************************************)

PROCEDURE SelectDisk( Drive : BYTE ) : BYTE;
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 0EH;
   Reg.DL := Drive;
   Lib.Dos( Reg );
   RETURN Reg.AL;               (* Number of drives *)
END SelectDisk;

PROCEDURE GetDrive() : SHORTCARD;
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 19H;
   Lib.Dos( Reg );
   RETURN Reg.AL+1;
END GetDrive;

PROCEDURE ChDir( VAR Directory : ARRAY OF CHAR  );
VAR
   Reg : SYSTEM.Registers;
BEGIN
   Reg.AH := 3BH; (* CHDIR *)
   Reg.DS := SYSTEM.Seg( Directory );
   Reg.DX := SYSTEM.Ofs( Directory );
   Lib.Dos( Reg );
END ChDir;

PROCEDURE ChDirDrv( VAR Directory : ARRAY OF CHAR );
VAR
   Drive            : CHAR;
   Disk, NoOfDrives : BYTE;
BEGIN
   Drive := Directory[0];
   Disk := BYTE( ORD( Drive ) - ORD( "A" ) );
   NoOfDrives := SelectDisk( Disk );
   ChDir( Directory );
END ChDirDrv;

PROCEDURE EnvironmentFind(     name   : ARRAY OF CHAR ;
                           VAR result : ARRAY OF CHAR ) ;
VAR
   n  : CARDINAL;
   pi : ARRAY [0..14] OF CHAR;
   es : ARRAY [0..255] OF CHAR;
   pp : Lib.CommandType;
BEGIN
   n := 0;
   LOOP
      pp := Lib.Environment( n );
      Str.Copy( es, pp^ );
      IF es[0] = CHR(0) THEN
         result[0] := CHR(0);
         RETURN;
      END;
      Str.ItemS( pi, es, '=', 0 );
      IF Str.Match( pi, name ) THEN
         Str.Copy( result, es );
         Str.Delete( result, 0, Str.Length( pi )+1 );
         RETURN;
      END;
      INC( n );
   END;
END EnvironmentFind;

PROCEDURE FindInPath( File : ARRAY OF CHAR; VAR FullName : ARRAY OF CHAR );
VAR
   Item          : CARDINAL;
   Found, NoMore : BOOLEAN;
   DosPath       : ARRAY [0..255] OF CHAR;
BEGIN
   Item   := 0;
   Found  := FALSE;
   NoMore := FALSE;
   EnvironmentFind( "PATH", DosPath );
   REPEAT
      Str.Item( FullName, DosPath, Str.CHARSET{';'}, Item );
      IF Str.Length( FullName ) = 0 THEN
         NoMore := TRUE;
      ELSIF FullName[Str.Length(FullName)-1] # '\' THEN
         Str.Append( FullName, '\' );
         Str.Append( FullName, File );
         Found := FIO.Exists( FullName );
      END;
      INC( Item );
   UNTIL Found OR NoMore;
END FindInPath;

PROCEDURE Exec( command, params : ARRAY OF CHAR ) : CARDINAL;
VAR
   memptr         : ADDRESS;
   memsize, reply : CARDINAL;
BEGIN
   memsize := Storage.HeapAvail( Storage.MainHeap )-8; (* 8 for MSDOS relocation *)
   Storage.HeapAllocate( Storage.MainHeap, memptr, memsize );
   reply := Lib.Execute( command, params, memptr, memsize );
   Storage.HeapDeallocate( Storage.MainHeap, memptr, memsize);
   RETURN reply;
END Exec;

PROCEDURE ExecCmd( command : ARRAY OF CHAR ) : CARDINAL;
VAR
   path  : ARRAY [0..63] OF CHAR;
   cline : ARRAY [0..128] OF CHAR;
BEGIN
   EnvironmentFind( 'COMSPEC', path );
   Str.Concat( cline, '/C ', command );
   RETURN Exec( path, cline );
END ExecCmd;

PROCEDURE ArchiveMethod( FileName : ARRAY OF CHAR ) : ArchiveType;
VAR
   Fin   : FIO.File;
   bytes : CARDINAL;
   Buf1  : ARRAY [0..24] OF CHAR;
BEGIN
   Fin := FIO.Open( FileName );
   bytes := FIO.RdBin( Fin, Buf1, SIZE(Buf1) );
   FIO.Close( Fin );
   IF (Buf1[0]='H') AND (Buf1[1]='L') AND
      (Buf1[2]='S') AND (Buf1[3]='Q') AND (Buf1[4]='Z') THEN
      RETURN SQZ;
   ELSIF (Buf1[0]='P') AND (Buf1[1]='K') AND
         (Buf1[2]=CHR(03H)) AND (Buf1[3]=CHR(04H)) THEN
      RETURN ZIP;
   ELSIF (Buf1[0]='H') AND (Buf1[1]='P') AND
         (Buf1[2]='A') AND (Buf1[3]='K') THEN
      RETURN HPK;
   ELSIF (Buf1[21]=CHR(0DCH)) AND (Buf1[22]=CHR(0A7H)) AND
         (Buf1[23]=CHR(0C4H)) AND (Buf1[24]=CHR(0FDH)) THEN
      RETURN ZOO;
   ELSIF (Buf1[2]='-') AND (Buf1[3]='l') AND (Buf1[6]='-') THEN
      RETURN LZH;
   ELSIF (Buf1[0]=CHR(060H)) AND (Buf1[1]=CHR(0EAH)) THEN
      RETURN ARJ;
   (* DWC, en alle ARC/PAK/A7P/HYP detecties nog inbouwen! *)
   ELSE
      RETURN none;
   END;
END ArchiveMethod;

PROCEDURE WrDosError( ErrorNo : SHORTCARD );
BEGIN
   CASE ErrorNo OF
     0    : IO.WrStr( "OK" );
   | 1    : IO.WrStr( "Invalid function number" );
   | 2    : IO.WrStr( "File not found" );
   | 3    : IO.WrStr( "Path not found" );
   | 4    : IO.WrStr( "Too many open files (no handles left)" );
   | 5    : IO.WrStr( "Access denied" );
   | 6    : IO.WrStr( "Invalid handle" );
   | 7    : IO.WrStr( "Memory control blocks destroyed" );
   | 8    : IO.WrStr( "Insufficient memory" );
   | 9    : IO.WrStr( "Invalid memory block address" );
   | 10   : IO.WrStr( "Invalid environment" );
   | 11   : IO.WrStr( "Invalid format" );
   | 12   : IO.WrStr( "Invalid access code" );
   | 13   : IO.WrStr( "Invalid data" );
   (* 14 : Reserved *)
   | 15   : IO.WrStr( "Invalid drive was specified" );
   | 16   : IO.WrStr( "Attempt to remove the current directory" );
   | 17   : IO.WrStr( "Not same device" );
   | 18   : IO.WrStr( "No more files" );
   | 19   : IO.WrStr( "Attempt to write on write-protected diskette" );
   | 20   : IO.WrStr( "Unknown unit" );
   | 21   : IO.WrStr( "Drive not ready" );
   | 22   : IO.WrStr( "Unknown command" );
   | 23   : IO.WrStr( "Data error (CRC)" );
   | 24   : IO.WrStr( "Bad request structure length" );
   | 25   : IO.WrStr( "Seek error" );
   | 26   : IO.WrStr( "Unknown media type" );
   | 27   : IO.WrStr( "Sector not found" );
   | 28   : IO.WrStr( "Printer out of paper" );
   | 29   : IO.WrStr( "Write fault" );
   | 30   : IO.WrStr( "Read fault" );
   | 31   : IO.WrStr( "General failure" );
   | 32   : IO.WrStr( "Sharing Violation" );
   | 33   : IO.WrStr( "Lock Violation" );
   | 34   : IO.WrStr( "Invalid disk change" );
   | 35   : IO.WrStr( "FCB unavailable" );
   (* 36..79 : Reserved *)
   | 80   : IO.WrStr( "File exists" );
   (* 81 : Reserved *)
   | 82   : IO.WrStr( "Cannot Make" );
   | 83   : IO.WrStr( "Fail on INT 24" );
   | 0F0H : IO.WrStr( "Disk Full (write failed)" ); (* JPI internal *)
   ELSE
      IO.WrStr( "Unknown DOS Error : " ); IO.WrShtCard( ErrorNo, 0 );
   END;
END WrDosError;

(***************************************************************************)
(* Time & Date Routines                                                    *)
(***************************************************************************)

TYPE
   Array12Type = ARRAY [1..12] OF CARDINAL;
CONST
   MonthDays = Array12Type(31,28,31,30,31,30,31,31,30,31,30,31);
   MonthCumm = Array12Type(0,31,59,90,120,151,181,212,243,273,304,334);

PROCEDURE GetTime( VAR Hrs, Mins, Secs, Hsecs : CARDINAL );
VAR
   Reg : SYSTEM.Registers;
BEGIN
  WITH Reg DO
    AH := 2CH;
    Lib.Dos( Reg );
    Hrs   := CARDINAL(CH);
    Mins  := CARDINAL(CL);
    Secs  := CARDINAL(DH);
    Hsecs := CARDINAL(DL);
  END;
END GetTime;

PROCEDURE GetDate( VAR Year, Month, Day : CARDINAL;
                   VAR DayOfWeek        : DayType );
VAR
   Reg : SYSTEM.Registers;
BEGIN
  WITH Reg DO
    AH := 2AH;
    Lib.Dos( Reg );
    Year      := CX;
    Month     := CARDINAL(DH);
    Day       := CARDINAL(DL);
    DayOfWeek := DayType(AL);
  END;
END GetDate;

PROCEDURE LeapYr( Year : CARDINAL ) : BOOLEAN;
BEGIN
   RETURN (Year MOD 4 = 0) AND ((Year MOD 100 <> 0) OR (Year MOD 400 = 0));
END LeapYr;

PROCEDURE DayNr( Year, Month, Day : CARDINAL ) : CARDINAL;
VAR
   i : CARDINAL;
BEGIN
   i := MonthCumm[Month]+Day;
   IF (Month > 2) AND LeapYr( Year ) THEN
      INC( i );
   END;
   RETURN i;
END DayNr;

PROCEDURE DayOfWeek( Year, Month, Day : CARDINAL ) : DayType;
VAR
   i        : CARDINAL;
   diff     : LONGCARD;
BEGIN
   diff := 0;
   FOR i := 1582 TO Year-1 DO
      diff := diff+365+LONGCARD(ORD( LeapYr( i ) ));
   END;
   RETURN DayType((diff+LONGCARD(DayNr( Year, Month, Day ))+4) MOD 7);
END DayOfWeek;

PROCEDURE WeekNr( Year, Month, Day : CARDINAL ) : CARDINAL;
VAR
   Week, Dow : CARDINAL;
BEGIN
   Dow := (ORD(DayOfWeek( Year, 1, 1 ))+6) MOD 7; (* Set switchdate to Monday *)
   Week := ((DayNr( Year, Month, Day )+Dow-1) DIV 7);
   IF Dow < 4 THEN
      INC( Week );
   END;
   IF Week = 0 THEN
      Week := WeekNr( Year-1, 12, 31 );
   END;
   RETURN Week;
END WeekNr;

(* Conversion Routines *)

PROCEDURE TimeToFileTime(     Hrs, Mins, Secs : CARDINAL;
                          VAR FileTime        : CARDINAL ) : BOOLEAN;
BEGIN
   FileTime := Hrs*2048+Mins*32+(Secs DIV 2);
   RETURN TRUE;
END TimeToFileTime;

PROCEDURE DateToFileDate(     Year, Month, Day : CARDINAL;
                          VAR FileDate         : CARDINAL ) : BOOLEAN;
BEGIN
   FileDate := (Year-1980)*512+Month*32+Day;
   RETURN TRUE;
END DateToFileDate;

PROCEDURE FileTimeToTime(     FileTime        : CARDINAL;
                          VAR Hrs, Mins, Secs : CARDINAL );
BEGIN
   Hrs  := FileTime DIV 2048;
   Mins := (FileTime MOD 2048) DIV 32;
   Secs := (FileTime MOD 32)*2;
END FileTimeToTime;

PROCEDURE FileDateToDate(     FileDate         : CARDINAL;
                          VAR Year, Month, Day : CARDINAL );
BEGIN
   Year  := (FileDate DIV 512)+1980;
   Month := (FileDate MOD 512) DIV 32;
   Day   := FileDate MOD 32;
END FileDateToDate;

PROCEDURE DateTimeToUts(     Year, Month, Day,
                             Hrs, Mins, Secs : CARDINAL;
                         VAR UTS             : LONGCARD ) : BOOLEAN;
VAR
   i        : CARDINAL;
   Status   : BOOLEAN;
BEGIN
   Status := TRUE;
   IF Year < 1970 THEN
      Status := FALSE
   ELSE
      UTS := 0;
      FOR i := 1970 TO Year-1 DO
         UTS := UTS+365+LONGCARD(ORD( LeapYr( i ) ));
      END;
      UTS := (UTS+LONGCARD(DayNr( Year, Month, Day ))-1)*86400
             +LONGCARD(Hrs)*3600+LONGCARD(Mins)*60+LONGCARD(Secs);
   END;
   RETURN Status;
END DateTimeToUts;

PROCEDURE UtsToDateTime(     UTS             : LONGCARD;
                         VAR Year, Month, Day,
                             Hrs, Mins, Secs : CARDINAL );
VAR
   days, Temp : LONGCARD;
BEGIN
   Secs := CARDINAL(UTS MOD 60);
   Mins := CARDINAL(UTS DIV 60 MOD 60);
   Hrs  := CARDINAL(UTS DIV 3600 MOD 24);
   days := UTS DIV 86400;
   Year := 1970;
   Temp := 365+LONGCARD(ORD(LeapYr( Year )));
   WHILE days >= Temp DO
      DEC( days, Temp );
      INC( Year );
      Temp := 365+LONGCARD(ORD(LeapYr( Year )));
   END;
   INC( days );   (* Daynumber now correct *)
   Month := 1;
   WHILE days > LONGCARD(MonthDays[Month]) DO
      DEC( days, LONGCARD(MonthDays[Month]) );
      IF (Month = 2) AND LeapYr( Year ) THEN
         DEC( days );
      END;
      INC( Month );
   END;
   IF days = 0 THEN
      days := 29;
      DEC( Month );
   END;
   Day := CARDINAL(days);
END UtsToDateTime;

PROCEDURE MonthToStr( Month : CARDINAL; VAR String : ARRAY OF CHAR );
BEGIN
   CASE Month OF
   |  1 : Str.Copy( String, "January" );
   |  2 : Str.Copy( String, "February" );
   |  3 : Str.Copy( String, "March" );
   |  4 : Str.Copy( String, "April" );
   |  5 : Str.Copy( String, "May" );
   |  6 : Str.Copy( String, "June" );
   |  7 : Str.Copy( String, "July" );
   |  8 : Str.Copy( String, "August" );
   |  9 : Str.Copy( String, "September" );
   | 10 : Str.Copy( String, "October" );
   | 11 : Str.Copy( String, "November" );
   | 12 : Str.Copy( String, "December" );
   ELSE
      Str.Copy( String, "UNKNOWN" );
   END;
END MonthToStr;

PROCEDURE DowToStr( DayOfWeek : DayType; VAR String : ARRAY OF CHAR );
BEGIN
   CASE DayOfWeek OF
   | Sunday    : Str.Copy( String, "Sunday" );
   | Monday    : Str.Copy( String, "Monday" );
   | Tuesday   : Str.Copy( String, "Tuesday" );
   | Wednesday : Str.Copy( String, "Wednesday" );
   | Thursday  : Str.Copy( String, "Thursday" );
   | Friday    : Str.Copy( String, "Friday" );
   | Saturday  : Str.Copy( String, "Saturday" );
   ELSE  Str.Copy( String, "UNKNOWN" );
   END;
END DowToStr;

PROCEDURE DateToStr(     Year, Month, Day : CARDINAL;
                         Format           : ARRAY OF CHAR;
                     VAR String           : ARRAY OF CHAR );
VAR
   Status  : BOOLEAN;
   I, J    : CARDINAL;
   TempStr : ARRAY [0..7] OF CHAR;
BEGIN
   String[0] := CHR(NUL);
   I := Str.Length( Format );
   J := 0;
   Status := TRUE;
   WHILE J < I  DO
      IF Format[J] = "%" THEN
         INC( J );
         IF J <= I THEN
            CASE Format[J] OF
            | "D" : Str.CardToStr( LONGCARD(Day), TempStr, 10, Status );
            | "M" : Str.CardToStr( LONGCARD(Month), TempStr, 10, Status );
            | "Y" : Str.CardToStr( LONGCARD(Year), TempStr, 10, Status );
            | "d" : Str.CardToStr( LONGCARD(Day), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            | "m" : Str.CardToStr( LONGCARD(Month), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            | "y" : Str.CardToStr( LONGCARD(Year MOD 100), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            | "s" : MonthToStr( Month, TempStr );
                    TempStr[3] := CHR( NUL );
            ELSE
               Status := FALSE;
            END;
            INC( J );
         ELSE
            Status := FALSE;
         END;
         Str.Append( String, TempStr );
      ELSE
         Str.Append( String, Format[J] );
         INC( J );
      END;
   END;
END DateToStr;

PROCEDURE TimeToStr(     Hour, Min, Sec : CARDINAL; Format : ARRAY OF CHAR;
                     VAR String : ARRAY OF CHAR );
VAR
   TempStr : ARRAY [0..7] OF CHAR;
   Status  : BOOLEAN;
   I, J    : CARDINAL;
BEGIN
   String[0] := CHR(NUL);
   I := Str.Length( Format );
   J := 0;
   Status := TRUE;
   WHILE J < I  DO
      IF Format[J] = "%" THEN
         INC( J );
         IF J <= I THEN
            CASE Format[J] OF
            | "H" : Str.CardToStr( LONGCARD(Hour), TempStr, 10, Status );
            | "M" : Str.CardToStr( LONGCARD(Min), TempStr, 10, Status );
            | "S" : Str.CardToStr( LONGCARD(Sec), TempStr, 10, Status );
            | "h" : Str.CardToStr( LONGCARD(Hour), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            | "m" : Str.CardToStr( LONGCARD(Min), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            | "s" : Str.CardToStr( LONGCARD(Sec), TempStr, 10, Status );
                    IF Str.Length( TempStr ) = 1 THEN
                       Str.Insert( TempStr, '0', 0 );
                    END;
            ELSE
               Status := FALSE;
            END;
            INC( J );
         ELSE
            Status := FALSE;
         END;
         Str.Append( String, TempStr );
      ELSE
         Str.Append( String, Format[J] );
         INC( J );
      END;
   END;
END TimeToStr;

(***************************************************************************)
(* Logging (using LogFiles)                                                *)
(***************************************************************************)

PROCEDURE LogDate( F : FIO.File );
VAR
   Year, Month, Day : CARDINAL;
   Dow              : DayType;
   DateString,
   LogString        : ARRAY [0..13] OF CHAR;
BEGIN
   GetDate( Year, Month, Day, Dow );
   DowToStr( Dow, LogString );
   LogString[3] := CHR(NUL);
   DateToStr( Year, Month, Day, " %d %s %y", DateString );
   Str.Append( LogString, DateString );
   FIO.WrStr( F, LogString );
END LogDate;

PROCEDURE LogTime( F : FIO.File );
VAR
   Hour, Min, Sec, Hsec : CARDINAL;
   LogString            : ARRAY [0..8] OF CHAR;
BEGIN
   GetTime( Hour, Min, Sec, Hsec );
   TimeToStr( Hour, Min, Sec, "%h:%m:%s", LogString );
   FIO.WrStr( F, LogString );
END LogTime;

PROCEDURE OpenLog( Name : ARRAY OF CHAR ) : FIO.File;
VAR
   F : FIO.File;
BEGIN
   IF FIO.Exists( Name ) THEN
      F := FIO.Append( Name );
   ELSE
      F := FIO.Create( Name );
   END;
   RETURN F;
END OpenLog;

PROCEDURE CloseLog( F : FIO.File );
BEGIN
   FIO.Close( F );
END CloseLog;

PROCEDURE LogHeader( LogFile, ProgID : ARRAY OF CHAR );
VAR
   F : FIO.File;
BEGIN
   F := OpenLog( LogFile );
   FIO.WrLn( F );
   FIO.WrStr( F, "----------  " );
   LogDate( F );
   FIO.WrStr( F, ", " );
   FIO.WrStr( F, ProgID );
   FIO.WrLn( F );
   CloseLog( F );
END LogHeader;

PROCEDURE LogEntry( LogFile, LogText : ARRAY OF CHAR; Level : CHAR );
VAR
   F : FIO.File;
BEGIN
   IF Level IN LogLevels THEN
      F := OpenLog( LogFile );
      FIO.WrChar( F, Level );
      FIO.WrChar( F, CHR(Space) );
      LogTime( F );
      FIO.WrStr( F, "  " );
      FIO.WrStr( F, LogText );
      FIO.WrLn( F );
      CloseLog( F );
   END;
END LogEntry;

(***************************************************************************)
(* Diversen                                                                *)
(***************************************************************************)

VAR
   Info : RECORD
      Length : CARDINAL;
      Line1a,
      Line1b,
      Line2a,
      Line2b : ARRAY [0..79] OF CHAR;
   END;

PROCEDURE SetProgramInfo( Len : CARDINAL; Name, Version, Date,
                          Programmer, Company : ARRAY OF CHAR   );
VAR
   SubStr : ARRAY [0..3] OF CHAR;
   Num    : CARDINAL;
   S      : ARRAY [0..8] OF CHAR;
   Status : BOOLEAN;
BEGIN
   Info.Length := Len;
   Str.Copy( Info.Line1a, Name );
   Str.Copy( Info.Line1b, Programmer );
   Str.Copy( Info.Line2a, "version " );
   Str.Append( Info.Line2a, Version );
   Str.Append( Info.Line2a, " (" );
   Str.Slice( SubStr, Date, 0, 2 );
   Num := CARDINAL( Str.StrToCard( SubStr, 10, Status ) );
   MonthToStr( Num, S );
   Str.Append( Info.Line2a, S );
   Str.Append( Info.Line2a, ", " );
   Str.Slice( SubStr, Date, 3, 2 );
   Num := CARDINAL( Str.StrToCard( SubStr, 10, Status ) );
   Str.CardToStr( LONGCARD( Num ), S, 10, Status );
   Str.Append( Info.Line2a, S );
   CASE Num OF
   | 1, 21, 31 :
      Str.Append( Info.Line2a, "st " );
   | 2, 22 :
      Str.Append( Info.Line2a, "nd " );
   | 3, 23 :
      Str.Append( Info.Line2a, "rd " );
   ELSE
      Str.Append( Info.Line2a, "th " );
   END;
   Str.Slice( SubStr, Date, 6, 4 );
   Str.Append( Info.Line2a, SubStr );
   Str.Append( Info.Line2a, ") " );
   Str.Copy( Info.Line2b, Company );
END SetProgramInfo;

PROCEDURE WriteProgramInfo();
BEGIN
   IO.WrCharRep( CHR(205), Info.Length );
   IO.WrLn;
   IO.WrStr( Info.Line1a );
   IO.WrStrAdj( Info.Line1b, Info.Length-Str.Length( Info.Line1a ) );
   IO.WrLn;
   IO.WrStr( Info.Line2a );
   IO.WrStrAdj( Info.Line2b, Info.Length-Str.Length( Info.Line2a ) );
   IO.WrLn;
   IO.WrCharRep( CHR(205), Info.Length );
   IO.WrLn;
   IO.WrLn;
END WriteProgramInfo;

END Tlib.
